//---------------------------------------------------------------------------


#pragma hdrstop

#include <strstrea>

#include "filetypes.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)


int wildcmp(const char *wild, const char *string) {
	const char *cp, *mp;

	while ((*string) && (*wild != '*')) {
		if ((toupper(*wild) != toupper(*string)) && (*wild != '?')) {
			return 0;
		}
		wild++;
		string++;
	}

	while (*string) {
		if (*wild == '*') {
			if (!*++wild) {
				return 1;
			}
			mp = wild;
			cp = string+1;
		} else if ((toupper(*wild) == toupper(*string)) || (*wild == '?')) {
			wild++;
			string++;
		} else {
			wild = mp;
			string = cp++;
		}
	}

	while (*wild == '*') {
		wild++;
	}
	return !*wild;
}



//---------------------------------------------------------------------------

TFileType::TFileType()
{
}

TFileType::TFileType(std::string n, std::string w)
  : name(n), wildcard(w),textno((TextEnum)0)
{
}

TFileType::TFileType(TextEnum n, std::string w)
  : textno(n), wildcard(w)
{
}



std::string TFileType::Name() const
{
  return (textno>0) ? LangText(textno).c_str(): name.c_str();
}

const std::string &TFileType::Wildcard() const
{
  return wildcard;
}

bool TFileType::Match(std::string s) const
{
  std::istrstream  is(wildcard.c_str());
  char buf[256];
  bool match=false;
  while (is.getline(buf,256,';'))
  {
    if (wildcmp(buf, s.c_str()))
      match=true;
  }
  return match;
}

void TFileType::Edit(std::string n, std::string w)
{
  name=n; wildcard=w;
}


bool TFileType::UserDefined() const
{
  return (textno==0);
}

//---------------------------------------------------------------------------


TFileTypes FileTypes;

TFileTypes::TFileTypes() : selected(0)
{
  Defaults();
}

void TFileTypes::Defaults()
{
   types.clear();
   types.push_back(TFileType(TE_All,"*"));
   types.push_back(TFileType(TE_Email,"mailto:*"));
   types.push_back(TFileType(TE_Web,"http:*;https:*"));
   types.push_back(TFileType(TE_FTP,"ftp:*"));
   types.push_back(TFileType(TE_Pictures,"*.jpg;*.jpeg;*.png;*.bmp;*.gif"));
   types.push_back(TFileType(TE_Movies,"*.mpg;*.mpeg;*.avi;*.asf"));
   types.push_back(TFileType(TE_Music,"*.mp3;*.wma;*.au;*.wav"));
   types.push_back(TFileType(TE_Archives,"*.zip;*.rar;*.arj;*.tgz;*.gz"));
}

const TFileType &TFileTypes::operator[](unsigned i) const
{
  return types.at(i);
}

unsigned TFileTypes::Size() const
{
  return types.size();
}

unsigned TFileTypes::Selected() const 
{
  return selected;
}

void TFileTypes::Select(unsigned i)
{
  selected=std::min(i,Size()-1);
}

bool TFileTypes::Match(std::string s) const 
{
  return types.at(selected).Match(s);
}



void TFileTypes::Add(TFileType t)
{
  types.push_back(t);
}

void TFileTypes::Remove(unsigned i)
{
  types.erase(types.begin()+i);
}

