//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <vector>
#include "setup.h"
#include "lang.h"
#include "winutil.h"
#include "filetypes.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TSetupForm *SetupForm;


//--------------------------------------------------------------------------
__fastcall TSetupForm::TSetupForm(TComponent* Owner)
        : TForm(Owner)
{
   UserUninstallOK=false;
   LangNotifier.AddLangNotify(this);
   LangChange();
}

void TSetupForm::LangChange()
{
   HotkeyLabel->Caption=LangText(TE_Hotkey);
   Caption=LangText(TE_Options);
   OKButton->Caption=LangText(TE_OK);
   CancelButton->Caption=LangText(TE_Cancel);
   StartAsGroup->Caption=LangText(TE_StartSetting);
   ActiveRadio->Caption=LangText(TE_Active);
   InactiveRadio->Caption=LangText(TE_Inactive);
   LastRadio->Caption=LangText(TE_LastSetting);
   HotkeyGroup->Caption=LangText(TE_HotkeyFunction);
   ConvertRadio->Caption=LangText(TE_ConvertNow);
   ToggleRadio->Caption=LangText(TE_ToggleActive);
   LangLabel->Caption=LangText(TE_Lang);
   const std::vector<AnsiString> &langs=LangList();
   LangCombo->Items->Clear();
   LangCombo->Items->Add(LangText(TE_AutoLang));
   for (unsigned i=0;i<langs.size();i++)
        LangCombo->Items->Add(langs[i]);
   LangCombo->ItemIndex=std::max(0,LangId2idx(GetRegInt("Language")));
   StripWSCheck->Caption      = LangText(TE_StripWS);
   StripCRRadio->Caption      = LangText(TE_StripCR);
   OnlyCopyLinksRadio->Caption= LangText(TE_OnlyCopyLinks);
   WindowsStartCheck->Caption = LangText(TE_StartWWin);
   UninstallCheck->Caption    = LangText(TE_Uninstall);
   HelpLabel->Caption         = LangText(TE_OnlineHelp);
   LoadFiletypes();
   LinkTypeLabel->Caption     = LangText(TE_LinkType);
   int index=LinkTypeCombo->ItemIndex;
   LinkTypeCombo->Items->Clear();
   LinkTypeEditor->Strings->Clear();
   for (unsigned i=0;i<FileTypes.Size();i++) {
     LinkTypeCombo->Items->Add(FileTypes[i].Name().c_str());
     LinkTypeEditor->Strings->Add(
          (FileTypes[i].Name()+"="+FileTypes[i].Wildcard()).c_str());
     LinkTypeEditor->ItemProps[i]->ReadOnly=!FileTypes[i].UserDefined();
   }
   LinkTypeCombo->ItemIndex=index;
   AddButton->Caption=LangText(TE_Add);
   DeleteButton->Caption=LangText(TE_Delete);
   TabGeneral->Caption       = LangText(TE_General);
   TabLinkTypes->Caption     = LangText(TE_LinkTypes);
   LinkTypeEditor->TitleCaptions->Strings[0] = LangText(TE_Name);
   LinkTypeEditor->TitleCaptions->Strings[1] = LangText(TE_Wildcard);
}

Classes::TShortCut __fastcall TSetupForm::GetHotKey()
{
   return GetRegInt("HotKey");
}



void __fastcall TSetupForm::FormShow(TObject *Sender)
{
   UserUninstallOK=false;
   HotKey1->HotKey=GetHotKey();
   ActiveRadio->Checked=false;
   InactiveRadio->Checked=false;
   LastRadio->Checked=false;
   switch(GetRegInt("StartAs"))
   {
      case 1: InactiveRadio->Checked=true; break;
      case 2: LastRadio->Checked=true; break;
      default: ActiveRadio->Checked=true; break;
   }
   if (GetRegInt("HotKeyFunction")==1)
      ConvertRadio->Checked=true;
   else
      ToggleRadio->Checked=true;


   LangCombo->ItemIndex=std::max(0,LangId2idx(GetRegInt("Language")));
   PrevLang=GetRegInt("Language");
   StripWSCheck->Checked=GetRegInt("StripWS");
   StripCRRadio->Checked=GetRegInt("StripCR");
   OnlyCopyLinksRadio->Checked=GetRegInt("OnlyCopyLinks");
   LinkTypeCombo->ItemIndex=GetRegInt("CopyLinkType");

   WindowsStartCheck->Checked=HasStartup();

   UninstallCheck->Checked=false;
   UserUninstallOK=true;

}
//---------------------------------------------------------------------------
void __fastcall TSetupForm::OKButtonClick(TObject *Sender)
{

   if (UninstallCheck->Checked)
   {
      Deinstall();
      RemoveStartup();
      ModalResult = mrOk;
   }
   else
   {
      short StartAs=-1;
      if (ActiveRadio->Checked) StartAs=0;
      else if (InactiveRadio->Checked) StartAs=1;
      else if (LastRadio->Checked) StartAs=2;
      if (ConvertRadio->Checked)
         SetRegInt("HotKeyFunction", 1);
      else if (ToggleRadio->Checked)
         SetRegInt("HotKeyFunction", 0);
      SetRegInt("StartAs", StartAs);
      SetRegInt("HotKey", HotKey1->HotKey);
      SetRegInt("StripWS", StripWSCheck->Checked);
      SetRegInt("StripCR", StripCRRadio->Checked);
      SetRegInt("Language", Name2LangId(LangCombo->Text));
      SetRegInt("OnlyCopyLinks", OnlyCopyLinksRadio->Checked);
      SetRegInt("CopyLinkType", LinkTypeCombo->ItemIndex);
      UpdateFiletypes();
      SaveFiletypes();

      if (WindowsStartCheck->Checked)
         InstallStartup();
      else
         RemoveStartup();
      ModalResult = mrOk;
   }
}
//---------------------------------------------------------------------------
void __fastcall TSetupForm::CancelButtonClick(TObject *Sender)
{
   if (Name2LangId(LangCombo->Text)!=PrevLang);{
      SetRegInt("Language", PrevLang);
      LangNotifier.LangChange();
   }
   LoadFiletypes();
   ModalResult = mrCancel;
}

void __fastcall TSetupForm::DeinstallAll()
{
}

void __fastcall TSetupForm::Deinstall()
{
  TRegistry *Registry = new TRegistry(KEY_READ);
  DeinstallUser(Registry);
}

void __fastcall TSetupForm::DeinstallUser(TRegistry *Registry)
{
  bool RemoveKlan=false;
  try
  {
    if (Registry->KeyExists("Software")) {
       Registry->OpenKey("Software",false);
       if (Registry->KeyExists("The Hovgaard Klan")) {
         Registry->OpenKey("The Hovgaard Klan",false);
         if (Registry->KeyExists("HovText")) {
            Registry->DeleteKey("HovText");
         }
         RemoveKlan=!Registry->HasSubKeys();
       }
    }
  }
  catch (...)
  {
  }
  delete Registry;
  if (RemoveKlan)
  {
    Registry = new TRegistry(KEY_READ);
    try
    {
       Registry->OpenKey("Software",false);
       Registry->DeleteKey("The Hovgaard Klan");
    }
    catch (...)
    {
    }
    delete Registry;
  }
}
//---------------------------------------------------------------------------



void __fastcall TSetupForm::UninstallCheckClick(TObject *Sender)
{
  if (UserUninstallOK)
  {
    if (!UninstallCheck->Checked)
      LoadFiletypes();;
    if (UninstallCheck->Checked &&
       (Application->MessageBox(LangText(TE_UninstallConfirm).c_str(),LangText(TE_Uninstall).c_str(),MB_YESNO	)==IDNO))
    {
      UserUninstallOK=false;
      UninstallCheck->Checked=false;
      UserUninstallOK=true;
    } else {
     ActiveRadio->Checked=false;
     InactiveRadio->Checked=false;
     LastRadio->Checked=false;
     ConvertRadio->Checked=false;
     ToggleRadio->Checked=false;
     LangCombo->ItemIndex=-1;
     StripWSCheck->Checked=false;
     StripCRRadio->Checked=false;
     OnlyCopyLinksRadio->Checked=false;
     WindowsStartCheck->Checked=false;
     HotKey1->HotKey=0;
     FileTypes.Defaults();
     LinkTypeEditor->Strings->Clear();
     for (unsigned i=0;i<FileTypes.Size();i++) {
       LinkTypeCombo->Items->Add(FileTypes[i].Name().c_str());
       LinkTypeEditor->Strings->Add(
            (FileTypes[i].Name()+"="+FileTypes[i].Wildcard()).c_str());
       LinkTypeEditor->ItemProps[i]->ReadOnly=!FileTypes[i].UserDefined();
     }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TSetupForm::URLLabelClick(TObject *Sender)
{
  ShellExecute(NULL,"open", URLLabel->Caption.c_str(),NULL,NULL,SW_RESTORE );
}
//---------------------------------------------------------------------------

void __fastcall TSetupForm::LangComboChange(TObject *Sender)
{
  SetRegInt("Language", Name2LangId(LangCombo->Text));
  LangNotifier.LangChange();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------

void __fastcall TSetupForm::SaveFiletypes()
{
  unsigned Count=GetRegInt("UserLinkCount");
  for (unsigned i=1;i<=Count;i++) {
    DeleteRegValue(AnsiString("UserLinkName")+i);
    DeleteRegValue(AnsiString("UserLinkWildcard")+i);
  }

  Count=0;
  for (unsigned i=0;i<FileTypes.Size();i++) {
   if (FileTypes[i].UserDefined())
   {
     Count++;
     SetRegString(AnsiString("UserLinkName")+Count, FileTypes[i].Name().c_str());
     SetRegString(AnsiString("UserLinkWildcard")+Count, FileTypes[i].Wildcard().c_str());
   }
  }
  SetRegInt("UserLinkCount",Count);
}

void __fastcall TSetupForm::LoadFiletypes()
{
  FileTypes.Defaults();
  unsigned Count=GetRegInt("UserLinkCount");
  for (unsigned i=1;i<=Count;i++) {
    FileTypes.Add(TFileType(GetRegString(AnsiString("UserLinkName")+i).c_str(),
                            GetRegString(AnsiString("UserLinkWildcard")+i).c_str()));

  }

}

void __fastcall TSetupForm::UpdateFiletypes()
{
  FileTypes.Defaults();
  for (int i=0;i<LinkTypeEditor->Strings->Count;i++)
  {
     if ((!LinkTypeEditor->ItemProps[i]->ReadOnly)
          && (LinkTypeEditor->Cells[0][i+1].Length()>0)
          && (LinkTypeEditor->Cells[1][i+1].Length()>0))
     {
        FileTypes.Add(TFileType(LinkTypeEditor->Cells[0][i+1].c_str(),
                                LinkTypeEditor->Cells[1][i+1].c_str()));
     }
  }
  int index=LinkTypeCombo->ItemIndex;
  LinkTypeCombo->Items->Clear();
  for (unsigned i=0;i<FileTypes.Size();i++) {
    LinkTypeCombo->Items->Add(FileTypes[i].Name().c_str());
  }
  LinkTypeCombo->ItemIndex=index;

}



void __fastcall TSetupForm::AddButtonClick(TObject *Sender)
{
for (int i=0;i<LinkTypeEditor->Strings->Count;i++)
  {
     if ((LinkTypeEditor->Cells[0][LinkTypeEditor->Strings->Count].Length()>0)
          && (LinkTypeEditor->Cells[1][LinkTypeEditor->Strings->Count].Length()>0))
     {
       LinkTypeEditor->Strings->Add("");
       LinkTypeEditor->Refresh();
     }
  }
}
//---------------------------------------------------------------------------

void __fastcall TSetupForm::DeleteButtonClick(TObject *Sender)
{
  if (!LinkTypeEditor->ItemProps[LinkTypeEditor->Row-1]->ReadOnly)
  {
    if (LinkTypeCombo->ItemIndex==LinkTypeEditor->Row-1)
      LinkTypeCombo->ItemIndex=0;
    LinkTypeEditor->Strings->Delete(LinkTypeEditor->Row-1);
    LinkTypeEditor->Row=LinkTypeEditor->Row-1;
    LinkTypeEditor->Refresh();
  } else
    Application->MessageBox(LangText(TE_CantDeletePredefined).c_str(),"HovText",MB_OK);
}
//---------------------------------------------------------------------------

void __fastcall TSetupForm::PageControl1Change(TObject *Sender)
{
  UpdateFiletypes();
}
//---------------------------------------------------------------------------

